from pathlib import Path

from extensions.chaeo.util import autonumber_new_directory, get_matching_files, loop_workflow
from extensions.chaeo.workflows import export_patches_from_multichannel_zstack

from model_server.accessors import CziImageFileAccessor, write_accessor_data_to_file, InMemoryDataAccessor
from model_server.process import rescale


def export_single_channel_tif_from_multichannel_czi(input_file_path, output_folder_path, channel, **kwargs):
    in_acc = CziImageFileAccessor(input_file_path)
    data = in_acc.get_one_channel_data(channel).data
    if 'rescale_zmask_clip' in kwargs:
        data = rescale(data, clip=kwargs['rescale_zmask_clip'])
    outf = Path(output_folder_path) / (Path(input_file_path).stem + '.tif')
    write_accessor_data_to_file(
        outf,
        InMemoryDataAccessor(data),
    )
    print(f'Wrote file: {outf}')

if __name__ == '__main__':
    where_czi = 'c:/Users/rhodes/projects/proj0012-trec-handoff/owncloud-sync/TREC-HD/Images/TREC_STOP_15_Kristineberg/230805_automic_AI_PA/20230805-122525_AI_PA_successfulrun_recognitiononPLL405cilindionas/Selection'
    where_output = autonumber_new_directory(
        'c:/Users/rhodes/projects/proj0011-plankton-seg/exp0016/output',
        'batch-output'
    )

    px_ilp = Path('c:/Users/rhodes/projects/proj0011-plankton-seg/exp0016/pxAF405_dim8bit.ilp')

    params = {
        'ilastik_project_file': px_ilp.__str__(),
        'pxmap_threshold': 0.25,
        'pixel_class': 0,
        'zmask_channel': 0,
        'patches_channel': 2,
        'mask_type': 'boxes',
        'zmask_filters': {'area': (1e3, 1e8)},
        'zmask_expand_box_by': (128, 3),
        'export_pixel_probabilities': True,
        'export_2d_patches_for_training': True,
        'export_2d_patches_for_annotation': True,
        'export_3d_patches': False,
        'export_annotated_zstack': True,
        'export_patch_masks': True,
        'rescale_zmask_clip': 0.01,
        'rgb_overlay_channels': (3, None, None),
        'rgb_overlay_weights': (0.5, 1.0, 1.0)
    }

    # input_files = get_matching_files(where_czi, 'czi', coord_filter={'P': (5, 10)},)
    input_files = get_matching_files(where_czi, 'czi', coord_filter={})


    tif_export_params = {
        'channel': 0,
        'rescale_zmask_clip': 0.01,
    }

    # loop_workflow(
    #     input_files,
    #     where_output,
    #     export_single_channel_tif_from_multichannel_czi,
    #     tif_export_params,
    #     catch_and_continue=False,
    #     export_batch_csvs=False,
    #     write_intermediate_products=False,
    # )

    loop_workflow(
        input_files,
        where_output,
        export_patches_from_multichannel_zstack,
        params,
        catch_and_continue=False,
    )

    print('Finished')